# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ...commons.types.facility_type_code import FacilityTypeCode
from ...commons.types.street_address_long_zip import StreetAddressLongZip
from ...custom_schemas.resources.v_1.types.schema_instance import SchemaInstance
from ...encounter_providers.resources.v_2.types.billing_provider_update import BillingProviderUpdate
from ...encounter_providers.resources.v_2.types.initial_referring_provider_update import InitialReferringProviderUpdate
from ...encounter_providers.resources.v_2.types.referring_provider_update import ReferringProviderUpdate
from ...encounter_providers.resources.v_2.types.rendering_provider_update import RenderingProviderUpdate
from ...encounter_providers.resources.v_2.types.supervising_provider_update import SupervisingProviderUpdate
from ...encounters.resources.v_4.types.claim_supplemental_information import ClaimSupplementalInformation
from ...encounters.resources.v_4.types.clinical_note_category_create import ClinicalNoteCategoryCreate
from ...encounters.resources.v_4.types.encounter_optional import EncounterOptional
from ...encounters.resources.v_4.types.epsdt_referral import EpsdtReferral
from ...encounters.resources.v_4.types.medication import Medication
from ...guarantor.resources.v_1.types.guarantor_update import GuarantorUpdate
from ...individual.types.patient_update import PatientUpdate
from ...individual.types.subscriber_create import SubscriberCreate
from ...property_and_casualty.resources.v_1.types.property_casualty_patient_identifier_update import (
    PropertyCasualtyPatientIdentifierUpdate,
)
from ...related_causes.resources.v_1.types.related_causes_information_update import RelatedCausesInformationUpdate
from ...service_facility.types.encounter_service_facility_update import EncounterServiceFacilityUpdate


class UniversalEncounterUpdateBase(EncounterOptional):
    epsdt_referral: typing.Optional[EpsdtReferral] = pydantic.Field(default=None)
    """
    Refers to Box 24H on the CMS1500 form and Loop 2300 CRC - EPSDT Referral on the 837P and 837i form
    """

    clinical_notes: typing.Optional[typing.List[ClinicalNoteCategoryCreate]] = pydantic.Field(default=None)
    """
    Holds a collection of clinical observations made by healthcare providers during patient encounters. Please note that medical records for appeals should be sent using the Encounter Attachments API.
    """

    claim_supplemental_information: typing.Optional[typing.List[ClaimSupplementalInformation]] = pydantic.Field(
        default=None
    )
    """
    Refers to Loop 2300 - Segment PWK on the 837P and 837i form. No more than 10 entries are permitted.
    """

    schema_instances: typing.Optional[typing.List[SchemaInstance]] = pydantic.Field(default=None)
    """
    Key-value pairs that must adhere to a schema created via the Custom Schema API. Multiple schema
    instances cannot be created for the same schema on an encounter. Updating schema instances utilizes PUT
    semantics, so the schema instances on the encounter will be set to whatever inputs are provided. If null
    is provided as an input, then the encounter's schema instances will be cleared.
    """

    existing_medications: typing.Optional[typing.List[Medication]] = pydantic.Field(default=None)
    """
    Existing medications that should be on the encounter.
    Note all current existing medications on encounter will be overridden with this list.
    """

    guarantor: typing.Optional[GuarantorUpdate] = pydantic.Field(default=None)
    """
    Personal and contact info for the guarantor of the patient responsibility.
    """

    subscriber_primary: typing.Optional[SubscriberCreate] = pydantic.Field(default=None)
    """
    Contains details of the primary insurance subscriber.
    """

    subscriber_secondary: typing.Optional[SubscriberCreate] = pydantic.Field(default=None)
    """
    Contains details of the secondary insurance subscriber.
    """

    subscriber_tertiary: typing.Optional[SubscriberCreate] = pydantic.Field(default=None)
    """
    Contains details of the tertiary insurance subscriber.
    """

    pay_to_address: typing.Optional[StreetAddressLongZip] = pydantic.Field(default=None)
    """
    Specifies the address to which payments for the claim should be sent.
    """

    initial_referring_provider: typing.Optional[InitialReferringProviderUpdate] = pydantic.Field(default=None)
    """
    The second iteration of Loop ID-2310 on an 837P form. Use code "P3 - Primary Care Provider" in this loop to
    indicate the initial referral from the primary care provider or whatever provider wrote the initial referral for this patient's episode of care being billed/reported in this transaction.
    Not used in an 837i claim
    """

    referring_provider: typing.Optional[ReferringProviderUpdate] = pydantic.Field(default=None)
    """
    The final provider who referred the services that were rendered.
    All physicians who order services or refer Medicare beneficiaries must
    report this data.
    """

    patient: typing.Optional[PatientUpdate] = pydantic.Field(default=None)
    """
    Contains the identification information of the individual receiving medical services.
    """

    rendering_provider: typing.Optional[RenderingProviderUpdate] = pydantic.Field(default=None)
    """
    The rendering provider is the practitioner -- physician, nurse practitioner, etc. -- performing the service.
    For telehealth services, the rendering provider performs the visit, asynchronous communication, or other service. The rendering provider address should generally be the same as the service facility address.
    """

    service_facility: typing.Optional[EncounterServiceFacilityUpdate] = pydantic.Field(default=None)
    """
    Encounter Service facility is typically the location a medical service was rendered, such as a provider office or hospital. For telehealth, service facility can represent the provider's location when the service was delivered (e.g., home), or the location where an in-person visit would have taken place, whichever is easier to identify. If the provider is in-network, service facility may be defined in payer contracts. Box 32 on the CMS-1500 claim form. There is no equivalent on the paper UB-04 claim form, but this field is equivalent to Loop 2310E Service Facility Location details on an 837i form, and is used when this is different to the entity identified as the Billing Provider. Note that for an in-network claim to be successfully adjudicated, the service facility address listed
    """

    supervising_provider: typing.Optional[SupervisingProviderUpdate] = pydantic.Field(default=None)
    """
    Required when the rendering provider is supervised by a physician. If not required by this implementation guide, do not send.
    """

    billing_provider: typing.Optional[BillingProviderUpdate] = pydantic.Field(default=None)
    """
    The billing provider is the provider or business entity submitting the claim. Billing provider may be, but is not necessarily, the same person/NPI as the rendering provider. From a payer's perspective, this represents the person or entity being reimbursed. When a contract exists with the target payer, the billing provider should be the entity contracted with the payer. In some circumstances, this will be an individual provider. In that case, submit that provider's NPI and the tax ID (TIN) that the provider gave to the payer during contracting. In other cases, the billing entity will be a medical group. If so, submit the group NPI and the group's tax ID. Box 33 on the CMS-1500 claim or Form Locator 1 on a UB-04 claim form.
    """

    place_of_service_code_as_submitted: typing.Optional[FacilityTypeCode] = pydantic.Field(default=None)
    """
    Box 24B on the CMS-1500 claim form. 837p Loop2300, CLM-05-1. This box is not used on a UB-04 or 837i claim form. 02 for telemedicine, 11 for in-person. Full list [here](https://www.cms.gov/Medicare/Coding/place-of-service-codes/Place_of_Service_Code_Set).
    """

    related_causes_information: typing.Optional[RelatedCausesInformationUpdate] = pydantic.Field(default=None)
    """
    Corresponds to box 10a on the CMS-1500 (Loop 2300 on 837)
    """

    property_casualty_claim_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    837p Loop2010 REF02, CMS1500 Box 11b
    """

    accident_date: typing.Optional[dt.date] = pydantic.Field(default=None)
    """
    837p Loop2300 DTP*439, CMS1500 Box 15
    """

    property_casualty_patient_identifier: typing.Optional[PropertyCasualtyPatientIdentifierUpdate] = pydantic.Field(
        default=None
    )
    """
    Patient identifier for Property and Casualty claims
    837p Loop 2010CA
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
from .....commons.types.resource_page import ResourcePage
from .organization_service_facility import OrganizationServiceFacility


class OrganizationServiceFacilityPage(ResourcePage):
    """
    Examples
    --------
    import uuid

    from candid.resources.commons import State, StreetAddressLongZip
    from candid.resources.organization_service_facilities.resources.v_2 import (
        OrganizationServiceFacility,
        OrganizationServiceFacilityPage,
        ServiceFacilityMode,
        ServiceFacilityOperationalStatus,
        ServiceFacilityPhysicalType,
        ServiceFacilityStatus,
        ServiceFacilityType,
    )

    OrganizationServiceFacilityPage(
        items=[
            OrganizationServiceFacility(
                organization_service_facility_id=uuid.UUID(
                    "30f55ee6-8c0e-43fc-a7fc-dac00d5bf569",
                ),
                name="Test Service Facility",
                aliases=["Test Service Facility Alias"],
                description="Test Service Facility Description",
                status=ServiceFacilityStatus.ACTIVE,
                operational_status=ServiceFacilityOperationalStatus.CLOSED,
                mode=ServiceFacilityMode.INSTANCE,
                type=ServiceFacilityType.DIAGNOSTICS_OR_THERAPEUTICS_UNIT,
                physical_type=ServiceFacilityPhysicalType.SITE,
                telecoms=["555-555-5555"],
                address=StreetAddressLongZip(
                    address_1="123 Main St",
                    address_2="Apt 1",
                    city="New York",
                    state=State.NY,
                    zip_code="10001",
                    zip_plus_four_code="1234",
                ),
            )
        ],
    )
    """

    items: typing.List[OrganizationServiceFacility]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ...billing_notes.resources.v_2.types.billing_note_base import BillingNoteBase
from ...claim_submission.resources.v_1.types.external_claim_submission_create import ExternalClaimSubmissionCreate
from ...custom_schemas.resources.v_1.types.schema_instance import SchemaInstance
from ...encounter_providers.resources.v_2.types.initial_referring_provider import InitialReferringProvider
from ...encounter_providers.resources.v_2.types.referring_provider import ReferringProvider
from ...encounter_providers.resources.v_2.types.supervising_provider import SupervisingProvider
from ...encounters.resources.v_4.types.claim_supplemental_information import ClaimSupplementalInformation
from ...encounters.resources.v_4.types.clinical_note_category_create import ClinicalNoteCategoryCreate
from ...encounters.resources.v_4.types.encounter_base import EncounterBase
from ...encounters.resources.v_4.types.epsdt_referral import EpsdtReferral
from ...encounters.resources.v_4.types.patient_history_category import PatientHistoryCategory
from ...encounters.resources.v_4.types.prior_authorization_number import PriorAuthorizationNumber
from ...encounters.resources.v_4.types.responsible_party_type import ResponsiblePartyType
from ...guarantor.resources.v_1.types.guarantor_create import GuarantorCreate
from ...individual.types.patient_create import PatientCreate
from ...individual.types.subscriber_create import SubscriberCreate
from ...property_and_casualty.resources.v_1.types.property_casualty_patient_identifier_create import (
    PropertyCasualtyPatientIdentifierCreate,
)
from ...related_causes.resources.v_1.types.related_causes_information_create import RelatedCausesInformationCreate
from ...service_facility.types.encounter_service_facility_base import EncounterServiceFacilityBase
from ...tags.types.tag_id import TagId


class UniversalEncounterCreateBase(EncounterBase):
    patient: PatientCreate = pydantic.Field()
    """
    Contains the identification information of the individual receiving medical services.
    """

    referring_provider: typing.Optional[ReferringProvider] = pydantic.Field(default=None)
    """
    The final provider who referred the services that were rendered.
    All physicians who order services or refer Medicare beneficiaries must
    report this data.
    """

    initial_referring_provider: typing.Optional[InitialReferringProvider] = pydantic.Field(default=None)
    """
    The second iteration of Loop ID-2310 on an 837P form. Use code "P3 - Primary Care Provider" in this loop to
    indicate the initial referral from the primary care provider or whatever provider wrote the initial referral for this patient's episode of care being billed/reported in this transaction.
    Not used in an 837i claim
    """

    supervising_provider: typing.Optional[SupervisingProvider] = pydantic.Field(default=None)
    """
    Required when the rendering provider is supervised by a physician. If not required by this implementation guide, do not send.
    """

    service_facility: typing.Optional[EncounterServiceFacilityBase] = pydantic.Field(default=None)
    """
    Encounter Service facility is typically the location a medical service was rendered, such as a provider office or hospital. For telehealth, service facility can represent the provider's location when the service was delivered (e.g., home), or the location where an in-person visit would have taken place, whichever is easier to identify. If the provider is in-network, service facility may be defined in payer contracts. Box 32 on the CMS-1500 claim form. There is no equivalent on the paper UB-04 claim form, but this field is equivalent to Loop 2310E Service Facility Location details on an 837i form, and is used when this is different to the entity identified as the Billing Provider. Note that for an in-network claim to be successfully adjudicated, the service facility address listed
    """

    subscriber_primary: typing.Optional[SubscriberCreate] = pydantic.Field(default=None)
    """
    Subscriber_primary is required when responsible_party is INSURANCE_PAY (i.e. when the claim should be billed to insurance).
    These are not required fields when responsible_party is SELF_PAY (i.e. when the claim should be billed to the patient).
    However, if you collect this for patients, even self-pay, we recommend including it when sending encounters to Candid.
    Note: Cash Pay is no longer a valid payer_id in v4, please use responsible party to define self-pay claims.
    """

    subscriber_secondary: typing.Optional[SubscriberCreate] = pydantic.Field(default=None)
    """
    Please always include this when you have it, even for self-pay claims.
    """

    subscriber_tertiary: typing.Optional[SubscriberCreate] = pydantic.Field(default=None)
    """
    Please always include this when you have it, even for self-pay claims.
    """

    prior_authorization_number: typing.Optional[PriorAuthorizationNumber] = pydantic.Field(default=None)
    """
    Box 23 on the CMS-1500 claim form or Form Locator 63 on a UB-04 claim form.
    """

    responsible_party: ResponsiblePartyType = pydantic.Field()
    """
    Defines the party to be billed with the initial balance owed on the claim.
    """

    clinical_notes: typing.Optional[typing.List[ClinicalNoteCategoryCreate]] = pydantic.Field(default=None)
    """
    Holds a collection of clinical observations made by healthcare providers during patient encounters. Please note that medical records for appeals should be sent using the Encounter Attachments API.
    """

    billing_notes: typing.Optional[typing.List[BillingNoteBase]] = pydantic.Field(default=None)
    """
    Spot to store misc, human-readable, notes about this encounter to be used
    in the billing process.
    """

    patient_histories: typing.Optional[typing.List[PatientHistoryCategory]] = None
    guarantor: typing.Optional[GuarantorCreate] = pydantic.Field(default=None)
    """
    Personal and contact info for the guarantor of the patient responsibility.
    """

    external_claim_submission: typing.Optional[ExternalClaimSubmissionCreate] = pydantic.Field(default=None)
    """
    To be included for claims that have been submitted outside of Candid.
    Candid supports posting remits and payments to these claims and working them in-platform (e.g. editing, resubmitting).
    """

    tag_ids: typing.Optional[typing.List[TagId]] = pydantic.Field(default=None)
    """
    Names of tags that should be on the encounter.
    """

    schema_instances: typing.Optional[typing.List[SchemaInstance]] = pydantic.Field(default=None)
    """
    Key-value pairs that must adhere to a schema created via the Custom Schema API. Multiple schema
    instances cannot be created for the same schema on an encounter.
    """

    referral_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    Refers to REF*9F on the 837p and 837i. Value cannot be greater than 50 characters.
    """

    epsdt_referral: typing.Optional[EpsdtReferral] = pydantic.Field(default=None)
    """
    Refers to Box 24H on the CMS1500 form and Loop 2300 CRC - EPSDT Referral on the 837P and 837i form
    """

    claim_supplemental_information: typing.Optional[typing.List[ClaimSupplementalInformation]] = pydantic.Field(
        default=None
    )
    """
    Refers to Loop 2300 - Segment PWK on the 837P and 837i forms. No more than 10 entries are permitted.
    """

    secondary_payer_carrier_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    When Medicaid is billed as the secondary payer the Carrier Code is used to identify the primary payer. This is required for certain states.
    """

    related_causes_information: typing.Optional[RelatedCausesInformationCreate] = pydantic.Field(default=None)
    """
    Corresponds to box 10a on the CMS-1500 (Loop 2300 on 837)
    """

    property_casualty_claim_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    837p Loop2010 REF02, CMS1500 Box 11b
    """

    accident_date: typing.Optional[dt.date] = pydantic.Field(default=None)
    """
    837p Loop2300 DTP*439, CMS1500 Box 15
    """

    property_casualty_patient_identifier: typing.Optional[PropertyCasualtyPatientIdentifierCreate] = pydantic.Field(
        default=None
    )
    """
    Patient identifier for Property and Casualty claims
    837p Loop 2010CA
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class PatientPaymentSource(enum.StrEnum):
    MANUAL_ENTRY = "MANUAL_ENTRY"
    CHARGEBEE_PAYMENTS = "CHARGEBEE_PAYMENTS"
    CHARGEBEE_MANUALLY_VOIDED_BY_CANDID = "CHARGEBEE MANUALLY VOIDED BY CANDID"
    CHARGEBEE_REFUNDS = "CHARGEBEE_REFUNDS"
    SQUARE_REFUNDS = "SQUARE_REFUNDS"
    SQUARE_PAYMENTS = "SQUARE_PAYMENTS"
    STRIPE_CHARGES = "STRIPE_CHARGES"
    STRIPE_REFUNDS = "STRIPE_REFUNDS"
    ELATION_PAYMENTS = "ELATION_PAYMENTS"
    _UNKNOWN = "__PATIENTPAYMENTSOURCE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "PatientPaymentSource":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        manual_entry: typing.Callable[[], T_Result],
        chargebee_payments: typing.Callable[[], T_Result],
        chargebee_manually_voided_by_candid: typing.Callable[[], T_Result],
        chargebee_refunds: typing.Callable[[], T_Result],
        square_refunds: typing.Callable[[], T_Result],
        square_payments: typing.Callable[[], T_Result],
        stripe_charges: typing.Callable[[], T_Result],
        stripe_refunds: typing.Callable[[], T_Result],
        elation_payments: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is PatientPaymentSource.MANUAL_ENTRY:
            return manual_entry()
        if self is PatientPaymentSource.CHARGEBEE_PAYMENTS:
            return chargebee_payments()
        if self is PatientPaymentSource.CHARGEBEE_MANUALLY_VOIDED_BY_CANDID:
            return chargebee_manually_voided_by_candid()
        if self is PatientPaymentSource.CHARGEBEE_REFUNDS:
            return chargebee_refunds()
        if self is PatientPaymentSource.SQUARE_REFUNDS:
            return square_refunds()
        if self is PatientPaymentSource.SQUARE_PAYMENTS:
            return square_payments()
        if self is PatientPaymentSource.STRIPE_CHARGES:
            return stripe_charges()
        if self is PatientPaymentSource.STRIPE_REFUNDS:
            return stripe_refunds()
        if self is PatientPaymentSource.ELATION_PAYMENTS:
            return elation_payments()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2
from .....common.types.base_model import BaseModel
from .....common.types.coverage_id import CoverageId
from .eligibility_recommendation_patient_info import EligibilityRecommendationPatientInfo
from .eligibility_recommendation_payload import EligibilityRecommendationPayload
from .vote import Vote


class EligibilityRecommendation(BaseModel):
    """
    An eligibility recommendation object that contains an EligibilityRecommendationType and a payload of data denoting the recommendation.
    """

    id: str = pydantic.Field()
    """
    The unique UUID identifier for an EligibilityRecommendation.
    """

    eligibility_check_id: str
    recommendation: EligibilityRecommendationPayload
    coverage_id: typing.Optional[CoverageId] = None
    patient: EligibilityRecommendationPatientInfo
    votes: typing.Optional[typing.List[Vote]] = pydantic.Field(default=None)
    """
    Array of votes for this recommendation
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .admitting_diagnosis import AdmittingDiagnosis
from .condition_information import ConditionInformation
from .diagnosis_related_group import DiagnosisRelatedGroup
from .external_cause_of_injury import ExternalCauseOfInjury
from .occurrence_information import OccurrenceInformation
from .occurrence_span_information import OccurrenceSpanInformation
from .other_diagnosis_information import OtherDiagnosisInformation
from .other_procedure_information import OtherProcedureInformation
from .patient_reason_for_visit import PatientReasonForVisit
from .principal_diagnosis import PrincipalDiagnosis
from .principal_procedure_information import PrincipalProcedureInformation
from .treatment_code_information import TreatmentCodeInformation
from .value_information import ValueInformation


class HealthCareCodeInformationGetAllResponse(UniversalBaseModel):
    principal_diagnosis: typing.Optional[PrincipalDiagnosis] = None
    other_diagnosis_information: typing.List[OtherDiagnosisInformation]
    admitting_diagnosis: typing.Optional[AdmittingDiagnosis] = pydantic.Field(default=None)
    """
    This object only applies to 837i institutional claim forms.
    """

    patient_reasons_for_visit: typing.List[PatientReasonForVisit] = pydantic.Field()
    """
    This object only applies to 837i institutional claim forms.
    """

    external_causes_of_injury: typing.List[ExternalCauseOfInjury] = pydantic.Field()
    """
    This object only applies to 837i institutional claim forms.
    """

    diagnosis_related_groups: typing.Optional[DiagnosisRelatedGroup] = pydantic.Field(default=None)
    """
    This object only applies to 837i institutional claim forms.
    """

    principal_procedure: typing.Optional[PrincipalProcedureInformation] = pydantic.Field(default=None)
    """
    This object only applies to 837i institutional claim forms.
    """

    other_procedure_information: typing.List[OtherProcedureInformation] = pydantic.Field()
    """
    This object only applies to 837i institutional claim forms.
    """

    occurrence_span_information: typing.List[OccurrenceSpanInformation] = pydantic.Field()
    """
    This object only applies to 837i institutional claim forms.
    """

    occurrence_information: typing.List[OccurrenceInformation] = pydantic.Field()
    """
    This object only applies to 837i institutional claim forms.
    """

    treatment_code_information: typing.List[TreatmentCodeInformation] = pydantic.Field()
    """
    This object only applies to 837i institutional claim forms.
    """

    value_information: typing.List[ValueInformation] = pydantic.Field()
    """
    This object only applies to 837i institutional claim forms.
    """

    condition_information: typing.List[ConditionInformation] = pydantic.Field()
    """
    This object only applies to 837i institutional claim forms.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class InvoiceDestination(enum.StrEnum):
    STRIPE = "STRIPE"
    CEDAR = "CEDAR"
    HEALTHIE = "HEALTHIE"
    COLLECTLY = "COLLECTLY"
    THIRD_PARTY_PAYERS = "THIRD_PARTY_PAYERS"
    INSTAMED = "INSTAMED"
    _UNKNOWN = "__INVOICEDESTINATION_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "InvoiceDestination":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        stripe: typing.Callable[[], T_Result],
        cedar: typing.Callable[[], T_Result],
        healthie: typing.Callable[[], T_Result],
        collectly: typing.Callable[[], T_Result],
        third_party_payers: typing.Callable[[], T_Result],
        instamed: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is InvoiceDestination.STRIPE:
            return stripe()
        if self is InvoiceDestination.CEDAR:
            return cedar()
        if self is InvoiceDestination.HEALTHIE:
            return healthie()
        if self is InvoiceDestination.COLLECTLY:
            return collectly()
        if self is InvoiceDestination.THIRD_PARTY_PAYERS:
            return third_party_payers()
        if self is InvoiceDestination.INSTAMED:
            return instamed()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from .......core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .......core.request_options import RequestOptions
from ....common.types.filter_query_string import FilterQueryString
from ....common.types.page_token import PageToken
from .raw_client import AsyncRawV1Client, RawV1Client
from .types.batch_eligibility_response import BatchEligibilityResponse
from .types.eligibility_check_page import EligibilityCheckPage
from .types.eligibility_recommendation import EligibilityRecommendation
from .types.eligibility_request import EligibilityRequest
from .types.eligibility_response import EligibilityResponse
from .types.payer_search_response import PayerSearchResponse
from .types.post_eligibility_recommendation_request import PostEligibilityRecommendationRequest
from .types.vote import Vote

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawV1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawV1Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawV1Client
        """
        return self._raw_client

    def post(
        self, *, request: EligibilityRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> EligibilityResponse:
        """
        Sends real-time eligibility checks to payers through Stedi.

        Parameters
        ----------
        request : EligibilityRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EligibilityResponse

        Examples
        --------
        from candid import CandidApiClient
        from candid.resources.pre_encounter.resources.coverages.resources.v_1 import (
            MemberInfo,
        )
        from candid.resources.pre_encounter.resources.eligibility_checks.resources.v_1 import (
            EligibilityRequest,
            IndividualProvider,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.eligibility_checks.v_1.post(
            request=EligibilityRequest(
                payer_id="payer_id",
                provider=IndividualProvider(
                    npi="npi",
                ),
                subscriber=MemberInfo(
                    first_name="first_name",
                    last_name="last_name",
                ),
            ),
        )
        """
        _response = self._raw_client.post(request=request, request_options=request_options)
        return _response.data

    def batch(
        self, *, request: typing.Sequence[EligibilityRequest], request_options: typing.Optional[RequestOptions] = None
    ) -> BatchEligibilityResponse:
        """
        Sends a batch of eligibility checks to payers through Stedi.

        Parameters
        ----------
        request : typing.Sequence[EligibilityRequest]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchEligibilityResponse

        Examples
        --------
        from candid import CandidApiClient
        from candid.resources.pre_encounter.resources.coverages.resources.v_1 import (
            MemberInfo,
        )
        from candid.resources.pre_encounter.resources.eligibility_checks.resources.v_1 import (
            EligibilityRequest,
            IndividualProvider,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.eligibility_checks.v_1.batch(
            request=[
                EligibilityRequest(
                    payer_id="payer_id",
                    provider=IndividualProvider(
                        npi="npi",
                    ),
                    subscriber=MemberInfo(
                        first_name="first_name",
                        last_name="last_name",
                    ),
                ),
                EligibilityRequest(
                    payer_id="payer_id",
                    provider=IndividualProvider(
                        npi="npi",
                    ),
                    subscriber=MemberInfo(
                        first_name="first_name",
                        last_name="last_name",
                    ),
                ),
            ],
        )
        """
        _response = self._raw_client.batch(request=request, request_options=request_options)
        return _response.data

    def poll_batch(
        self,
        batch_id: str,
        *,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EligibilityCheckPage:
        """
        Polls the status of a batch eligibility check.

        Parameters
        ----------
        batch_id : str

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EligibilityCheckPage

        Examples
        --------
        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.eligibility_checks.v_1.poll_batch(
            batch_id="batch_id",
        )
        """
        _response = self._raw_client.poll_batch(batch_id, page_token=page_token, request_options=request_options)
        return _response.data

    def payer_search(
        self,
        *,
        page_size: typing.Optional[int] = None,
        page_token: typing.Optional[PageToken] = None,
        query: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayerSearchResponse:
        """
        Searches for payers that match the query parameters.

        Parameters
        ----------
        page_size : typing.Optional[int]

        page_token : typing.Optional[PageToken]

        query : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayerSearchResponse

        Examples
        --------
        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.eligibility_checks.v_1.payer_search()
        """
        _response = self._raw_client.payer_search(
            page_size=page_size, page_token=page_token, query=query, request_options=request_options
        )
        return _response.data

    def recommendation(
        self,
        *,
        filters: typing.Optional[FilterQueryString] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[EligibilityRecommendation]:
        """
        Gets recommendation for eligibility checks based on filters. This endpoint will retrieve all the latest eligibility recommendations for each
        eligibility recommendation type for the given filters. If you want to get a specific recommendation type, you can use the `type` query parameter.

        Parameters
        ----------
        filters : typing.Optional[FilterQueryString]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[EligibilityRecommendation]

        Examples
        --------
        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.eligibility_checks.v_1.recommendation()
        """
        _response = self._raw_client.recommendation(filters=filters, request_options=request_options)
        return _response.data

    def create_recommendation(
        self, *, request: PostEligibilityRecommendationRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> EligibilityRecommendation:
        """
        Create an eligibiilty recommendation based on the request.

        Parameters
        ----------
        request : PostEligibilityRecommendationRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EligibilityRecommendation

        Examples
        --------
        from candid import CandidApiClient
        from candid.resources.pre_encounter.resources.eligibility_checks.resources.v_1 import (
            EligibilityRecommendationPatientInfo,
            EligibilityRecommendationPayload_MedicareAdvantage,
            MedicareAdvantageRecommendationPayload,
            PostEligibilityRecommendationRequest,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.eligibility_checks.v_1.create_recommendation(
            request=PostEligibilityRecommendationRequest(
                eligibility_check_id="eligibility_check_id",
                patient=EligibilityRecommendationPatientInfo(),
                recommendation=EligibilityRecommendationPayload_MedicareAdvantage(
                    payload=MedicareAdvantageRecommendationPayload(),
                ),
            ),
        )
        """
        _response = self._raw_client.create_recommendation(request=request, request_options=request_options)
        return _response.data

    def vote_recommendation(
        self,
        recommendation_id: str,
        version: str,
        *,
        request: Vote,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EligibilityRecommendation:
        """
        Submit user feedback on an eligibility recommendation. The path must contain the next version number to prevent race conditions. For example, if the current version of the recommendation is n, you will need to send a request to this endpoint with `/{recommendation_id}/{n+1}/vote` to update the vote.

        Parameters
        ----------
        recommendation_id : str

        version : str

        request : Vote

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EligibilityRecommendation

        Examples
        --------
        from candid import CandidApiClient
        from candid.resources.pre_encounter.resources.eligibility_checks.resources.v_1 import (
            Vote,
            VoteValue,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.eligibility_checks.v_1.vote_recommendation(
            recommendation_id="recommendation_id",
            version="version",
            request=Vote(
                user_id="user_id",
                value=VoteValue.UPVOTE,
            ),
        )
        """
        _response = self._raw_client.vote_recommendation(
            recommendation_id, version, request=request, request_options=request_options
        )
        return _response.data

    def get_multi(
        self,
        *,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        subscriber_member_id: typing.Optional[str] = None,
        payer_id: typing.Optional[str] = None,
        provider_npi: typing.Optional[str] = None,
        date_of_service: typing.Optional[str] = None,
        initiated_at_min: typing.Optional[dt.datetime] = None,
        initiated_at_max: typing.Optional[dt.datetime] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EligibilityCheckPage:
        """
        Parameters
        ----------
        page_token : typing.Optional[PageToken]

        limit : typing.Optional[int]

        subscriber_member_id : typing.Optional[str]

        payer_id : typing.Optional[str]

        provider_npi : typing.Optional[str]

        date_of_service : typing.Optional[str]

        initiated_at_min : typing.Optional[dt.datetime]

        initiated_at_max : typing.Optional[dt.datetime]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EligibilityCheckPage

        Examples
        --------
        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.eligibility_checks.v_1.get_multi()
        """
        _response = self._raw_client.get_multi(
            page_token=page_token,
            limit=limit,
            subscriber_member_id=subscriber_member_id,
            payer_id=payer_id,
            provider_npi=provider_npi,
            date_of_service=date_of_service,
            initiated_at_min=initiated_at_min,
            initiated_at_max=initiated_at_max,
            request_options=request_options,
        )
        return _response.data


class AsyncV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawV1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawV1Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawV1Client
        """
        return self._raw_client

    async def post(
        self, *, request: EligibilityRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> EligibilityResponse:
        """
        Sends real-time eligibility checks to payers through Stedi.

        Parameters
        ----------
        request : EligibilityRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EligibilityResponse

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient
        from candid.resources.pre_encounter.resources.coverages.resources.v_1 import (
            MemberInfo,
        )
        from candid.resources.pre_encounter.resources.eligibility_checks.resources.v_1 import (
            EligibilityRequest,
            IndividualProvider,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.eligibility_checks.v_1.post(
                request=EligibilityRequest(
                    payer_id="payer_id",
                    provider=IndividualProvider(
                        npi="npi",
                    ),
                    subscriber=MemberInfo(
                        first_name="first_name",
                        last_name="last_name",
                    ),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.post(request=request, request_options=request_options)
        return _response.data

    async def batch(
        self, *, request: typing.Sequence[EligibilityRequest], request_options: typing.Optional[RequestOptions] = None
    ) -> BatchEligibilityResponse:
        """
        Sends a batch of eligibility checks to payers through Stedi.

        Parameters
        ----------
        request : typing.Sequence[EligibilityRequest]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchEligibilityResponse

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient
        from candid.resources.pre_encounter.resources.coverages.resources.v_1 import (
            MemberInfo,
        )
        from candid.resources.pre_encounter.resources.eligibility_checks.resources.v_1 import (
            EligibilityRequest,
            IndividualProvider,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.eligibility_checks.v_1.batch(
                request=[
                    EligibilityRequest(
                        payer_id="payer_id",
                        provider=IndividualProvider(
                            npi="npi",
                        ),
                        subscriber=MemberInfo(
                            first_name="first_name",
                            last_name="last_name",
                        ),
                    ),
                    EligibilityRequest(
                        payer_id="payer_id",
                        provider=IndividualProvider(
                            npi="npi",
                        ),
                        subscriber=MemberInfo(
                            first_name="first_name",
                            last_name="last_name",
                        ),
                    ),
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.batch(request=request, request_options=request_options)
        return _response.data

    async def poll_batch(
        self,
        batch_id: str,
        *,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EligibilityCheckPage:
        """
        Polls the status of a batch eligibility check.

        Parameters
        ----------
        batch_id : str

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EligibilityCheckPage

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.eligibility_checks.v_1.poll_batch(
                batch_id="batch_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.poll_batch(batch_id, page_token=page_token, request_options=request_options)
        return _response.data

    async def payer_search(
        self,
        *,
        page_size: typing.Optional[int] = None,
        page_token: typing.Optional[PageToken] = None,
        query: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayerSearchResponse:
        """
        Searches for payers that match the query parameters.

        Parameters
        ----------
        page_size : typing.Optional[int]

        page_token : typing.Optional[PageToken]

        query : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayerSearchResponse

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.eligibility_checks.v_1.payer_search()


        asyncio.run(main())
        """
        _response = await self._raw_client.payer_search(
            page_size=page_size, page_token=page_token, query=query, request_options=request_options
        )
        return _response.data

    async def recommendation(
        self,
        *,
        filters: typing.Optional[FilterQueryString] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[EligibilityRecommendation]:
        """
        Gets recommendation for eligibility checks based on filters. This endpoint will retrieve all the latest eligibility recommendations for each
        eligibility recommendation type for the given filters. If you want to get a specific recommendation type, you can use the `type` query parameter.

        Parameters
        ----------
        filters : typing.Optional[FilterQueryString]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[EligibilityRecommendation]

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.eligibility_checks.v_1.recommendation()


        asyncio.run(main())
        """
        _response = await self._raw_client.recommendation(filters=filters, request_options=request_options)
        return _response.data

    async def create_recommendation(
        self, *, request: PostEligibilityRecommendationRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> EligibilityRecommendation:
        """
        Create an eligibiilty recommendation based on the request.

        Parameters
        ----------
        request : PostEligibilityRecommendationRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EligibilityRecommendation

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient
        from candid.resources.pre_encounter.resources.eligibility_checks.resources.v_1 import (
            EligibilityRecommendationPatientInfo,
            EligibilityRecommendationPayload_MedicareAdvantage,
            MedicareAdvantageRecommendationPayload,
            PostEligibilityRecommendationRequest,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.eligibility_checks.v_1.create_recommendation(
                request=PostEligibilityRecommendationRequest(
                    eligibility_check_id="eligibility_check_id",
                    patient=EligibilityRecommendationPatientInfo(),
                    recommendation=EligibilityRecommendationPayload_MedicareAdvantage(
                        payload=MedicareAdvantageRecommendationPayload(),
                    ),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_recommendation(request=request, request_options=request_options)
        return _response.data

    async def vote_recommendation(
        self,
        recommendation_id: str,
        version: str,
        *,
        request: Vote,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EligibilityRecommendation:
        """
        Submit user feedback on an eligibility recommendation. The path must contain the next version number to prevent race conditions. For example, if the current version of the recommendation is n, you will need to send a request to this endpoint with `/{recommendation_id}/{n+1}/vote` to update the vote.

        Parameters
        ----------
        recommendation_id : str

        version : str

        request : Vote

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EligibilityRecommendation

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient
        from candid.resources.pre_encounter.resources.eligibility_checks.resources.v_1 import (
            Vote,
            VoteValue,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.eligibility_checks.v_1.vote_recommendation(
                recommendation_id="recommendation_id",
                version="version",
                request=Vote(
                    user_id="user_id",
                    value=VoteValue.UPVOTE,
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.vote_recommendation(
            recommendation_id, version, request=request, request_options=request_options
        )
        return _response.data

    async def get_multi(
        self,
        *,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        subscriber_member_id: typing.Optional[str] = None,
        payer_id: typing.Optional[str] = None,
        provider_npi: typing.Optional[str] = None,
        date_of_service: typing.Optional[str] = None,
        initiated_at_min: typing.Optional[dt.datetime] = None,
        initiated_at_max: typing.Optional[dt.datetime] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EligibilityCheckPage:
        """
        Parameters
        ----------
        page_token : typing.Optional[PageToken]

        limit : typing.Optional[int]

        subscriber_member_id : typing.Optional[str]

        payer_id : typing.Optional[str]

        provider_npi : typing.Optional[str]

        date_of_service : typing.Optional[str]

        initiated_at_min : typing.Optional[dt.datetime]

        initiated_at_max : typing.Optional[dt.datetime]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EligibilityCheckPage

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.eligibility_checks.v_1.get_multi()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_multi(
            page_token=page_token,
            limit=limit,
            subscriber_member_id=subscriber_member_id,
            payer_id=payer_id,
            provider_npi=provider_npi,
            date_of_service=date_of_service,
            initiated_at_min=initiated_at_min,
            initiated_at_max=initiated_at_max,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .denial_reason_content import DenialReasonContent
    from .drug_identification import DrugIdentification
    from .drug_identification_optional import DrugIdentificationOptional
    from .measurement_unit_code import MeasurementUnitCode
    from .service_id_qualifier import ServiceIdQualifier
    from .service_line import ServiceLine
    from .service_line_adjustment import ServiceLineAdjustment
    from .service_line_create import ServiceLineCreate
    from .service_line_create_base import ServiceLineCreateBase
    from .service_line_create_base_base import ServiceLineCreateBaseBase
    from .service_line_create_optional import ServiceLineCreateOptional
    from .service_line_create_standalone import ServiceLineCreateStandalone
    from .service_line_create_standalone_base import ServiceLineCreateStandaloneBase
    from .service_line_create_standalone_base_base import ServiceLineCreateStandaloneBaseBase
    from .service_line_denial_reason import ServiceLineDenialReason
    from .service_line_era_data import ServiceLineEraData
    from .service_line_update import ServiceLineUpdate
    from .service_line_update_base import ServiceLineUpdateBase
    from .test_result import TestResult
    from .test_result_optional import TestResultOptional
    from .test_result_type import TestResultType
    from .universal_service_line_create import UniversalServiceLineCreate
    from .universal_service_line_create_standalone import UniversalServiceLineCreateStandalone
    from .universal_service_line_update import UniversalServiceLineUpdate
_dynamic_imports: typing.Dict[str, str] = {
    "DenialReasonContent": ".denial_reason_content",
    "DrugIdentification": ".drug_identification",
    "DrugIdentificationOptional": ".drug_identification_optional",
    "MeasurementUnitCode": ".measurement_unit_code",
    "ServiceIdQualifier": ".service_id_qualifier",
    "ServiceLine": ".service_line",
    "ServiceLineAdjustment": ".service_line_adjustment",
    "ServiceLineCreate": ".service_line_create",
    "ServiceLineCreateBase": ".service_line_create_base",
    "ServiceLineCreateBaseBase": ".service_line_create_base_base",
    "ServiceLineCreateOptional": ".service_line_create_optional",
    "ServiceLineCreateStandalone": ".service_line_create_standalone",
    "ServiceLineCreateStandaloneBase": ".service_line_create_standalone_base",
    "ServiceLineCreateStandaloneBaseBase": ".service_line_create_standalone_base_base",
    "ServiceLineDenialReason": ".service_line_denial_reason",
    "ServiceLineEraData": ".service_line_era_data",
    "ServiceLineUpdate": ".service_line_update",
    "ServiceLineUpdateBase": ".service_line_update_base",
    "TestResult": ".test_result",
    "TestResultOptional": ".test_result_optional",
    "TestResultType": ".test_result_type",
    "UniversalServiceLineCreate": ".universal_service_line_create",
    "UniversalServiceLineCreateStandalone": ".universal_service_line_create_standalone",
    "UniversalServiceLineUpdate": ".universal_service_line_update",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "DenialReasonContent",
    "DrugIdentification",
    "DrugIdentificationOptional",
    "MeasurementUnitCode",
    "ServiceIdQualifier",
    "ServiceLine",
    "ServiceLineAdjustment",
    "ServiceLineCreate",
    "ServiceLineCreateBase",
    "ServiceLineCreateBaseBase",
    "ServiceLineCreateOptional",
    "ServiceLineCreateStandalone",
    "ServiceLineCreateStandaloneBase",
    "ServiceLineCreateStandaloneBaseBase",
    "ServiceLineDenialReason",
    "ServiceLineEraData",
    "ServiceLineUpdate",
    "ServiceLineUpdateBase",
    "TestResult",
    "TestResultOptional",
    "TestResultType",
    "UniversalServiceLineCreate",
    "UniversalServiceLineCreateStandalone",
    "UniversalServiceLineUpdate",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
from .....organization_service_facilities.resources.v_2.types.organization_service_facility import (
    OrganizationServiceFacility,
)
from .base_credentialing_span import BaseCredentialingSpan
from .facility_credentialing_span_id import FacilityCredentialingSpanId


class FacilityCredentialingSpan(BaseCredentialingSpan):
    facility_credentialing_span_id: FacilityCredentialingSpanId
    service_facility: OrganizationServiceFacility = pydantic.Field()
    """
    The service facility covered by the credentialing span.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .non_insurance_payer_address_update import NonInsurancePayerAddressUpdate
from .non_insurance_payer_category_update import NonInsurancePayerCategoryUpdate
from .non_insurance_payer_description_update import NonInsurancePayerDescriptionUpdate


class NonInsurancePayerUpdateRequest(UniversalBaseModel):
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Max 50 characters allowed
    """

    description: typing.Optional[NonInsurancePayerDescriptionUpdate] = None
    category: typing.Optional[NonInsurancePayerCategoryUpdate] = None
    address: typing.Optional[NonInsurancePayerAddressUpdate] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.http_response import AsyncHttpResponse, HttpResponse
from .....core.jsonable_encoder import jsonable_encoder
from .....core.pydantic_utilities import parse_obj_as
from .....core.request_options import RequestOptions
from ....commons.errors.entity_not_found_error import EntityNotFoundError
from ....commons.errors.unauthorized_error import UnauthorizedError
from ....commons.errors.unprocessable_entity_error import UnprocessableEntityError
from ....commons.types.claim_id import ClaimId
from ....commons.types.entity_not_found_error_message import EntityNotFoundErrorMessage
from ....commons.types.invoice_id import InvoiceId
from ....commons.types.page_token import PageToken
from ....commons.types.patient_external_id import PatientExternalId
from ....commons.types.provider_id import ProviderId
from ....commons.types.service_line_id import ServiceLineId
from ....commons.types.sort_direction import SortDirection
from ....commons.types.unauthorized_error_message import UnauthorizedErrorMessage
from ....commons.types.unprocessable_entity_error_message import UnprocessableEntityErrorMessage
from ....financials.types.allocation_create import AllocationCreate
from ....financials.types.invoice_update import InvoiceUpdate
from ....financials.types.note_update import NoteUpdate
from ....financials.types.patient_transaction_source import PatientTransactionSource
from ....financials.types.refund_reason import RefundReason
from ....financials.types.refund_reason_update import RefundReasonUpdate
from .types.patient_refund import PatientRefund
from .types.patient_refund_id import PatientRefundId
from .types.patient_refund_sort_field import PatientRefundSortField
from .types.patient_refunds_page import PatientRefundsPage

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawV1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        patient_external_id: typing.Optional[PatientExternalId] = None,
        claim_id: typing.Optional[ClaimId] = None,
        service_line_id: typing.Optional[ServiceLineId] = None,
        billing_provider_id: typing.Optional[ProviderId] = None,
        unattributed: typing.Optional[bool] = None,
        invoice_id: typing.Optional[InvoiceId] = None,
        sources: typing.Optional[
            typing.Union[PatientTransactionSource, typing.Sequence[PatientTransactionSource]]
        ] = None,
        sort: typing.Optional[PatientRefundSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PatientRefundsPage]:
        """
        Returns all patient refunds satisfying the search criteria AND whose organization_id matches
        the current organization_id of the authenticated user.

        Parameters
        ----------
        limit : typing.Optional[int]
            Defaults to 100. The value must be greater than 0 and less than 1000.

        patient_external_id : typing.Optional[PatientExternalId]

        claim_id : typing.Optional[ClaimId]

        service_line_id : typing.Optional[ServiceLineId]

        billing_provider_id : typing.Optional[ProviderId]

        unattributed : typing.Optional[bool]
            returns payments with unattributed allocations if set to true

        invoice_id : typing.Optional[InvoiceId]

        sources : typing.Optional[typing.Union[PatientTransactionSource, typing.Sequence[PatientTransactionSource]]]

        sort : typing.Optional[PatientRefundSortField]
            Defaults to refund_timestamp

        sort_direction : typing.Optional[SortDirection]
            Sort direction. Defaults to descending order if not provided.

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PatientRefundsPage]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/patient-refunds/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "limit": limit,
                "patient_external_id": patient_external_id,
                "claim_id": claim_id,
                "service_line_id": service_line_id,
                "billing_provider_id": billing_provider_id,
                "unattributed": unattributed,
                "invoice_id": invoice_id,
                "sources": sources,
                "sort": sort,
                "sort_direction": sort_direction,
                "page_token": page_token,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PatientRefundsPage,
                parse_obj_as(
                    type_=PatientRefundsPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, patient_refund_id: PatientRefundId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PatientRefund]:
        """
        Retrieves a previously created patient refund by its `patient_refund_id`.

        Parameters
        ----------
        patient_refund_id : PatientRefundId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PatientRefund]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/patient-refunds/v1/{jsonable_encoder(patient_refund_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PatientRefund,
                parse_obj_as(
                    type_=PatientRefund,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        amount_cents: int,
        patient_external_id: PatientExternalId,
        allocations: typing.Sequence[AllocationCreate],
        refund_timestamp: typing.Optional[dt.datetime] = OMIT,
        refund_note: typing.Optional[str] = OMIT,
        invoice: typing.Optional[InvoiceId] = OMIT,
        refund_reason: typing.Optional[RefundReason] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PatientRefund]:
        """
        Creates a new patient refund record and returns the newly created PatientRefund object.
        The allocations can describe whether the refund is being applied toward a specific service line,
        claim, or billing provider.

        Parameters
        ----------
        amount_cents : int

        patient_external_id : PatientExternalId

        allocations : typing.Sequence[AllocationCreate]

        refund_timestamp : typing.Optional[dt.datetime]

        refund_note : typing.Optional[str]

        invoice : typing.Optional[InvoiceId]

        refund_reason : typing.Optional[RefundReason]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PatientRefund]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/patient-refunds/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json={
                "amount_cents": amount_cents,
                "refund_timestamp": refund_timestamp,
                "refund_note": refund_note,
                "patient_external_id": patient_external_id,
                "allocations": allocations,
                "invoice": invoice,
                "refund_reason": refund_reason,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PatientRefund,
                parse_obj_as(
                    type_=PatientRefund,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        patient_refund_id: PatientRefundId,
        *,
        refund_timestamp: typing.Optional[dt.datetime] = OMIT,
        refund_note: typing.Optional[NoteUpdate] = OMIT,
        invoice: typing.Optional[InvoiceUpdate] = OMIT,
        refund_reason: typing.Optional[RefundReasonUpdate] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PatientRefund]:
        """
        Updates the patient refund record matching the provided patient_refund_id.

        Parameters
        ----------
        patient_refund_id : PatientRefundId

        refund_timestamp : typing.Optional[dt.datetime]

        refund_note : typing.Optional[NoteUpdate]

        invoice : typing.Optional[InvoiceUpdate]

        refund_reason : typing.Optional[RefundReasonUpdate]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PatientRefund]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/patient-refunds/v1/{jsonable_encoder(patient_refund_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json={
                "refund_timestamp": refund_timestamp,
                "refund_note": refund_note,
                "invoice": invoice,
                "refund_reason": refund_reason,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PatientRefund,
                parse_obj_as(
                    type_=PatientRefund,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self, patient_refund_id: PatientRefundId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Deletes the patient refund record matching the provided patient_refund_id.

        Parameters
        ----------
        patient_refund_id : PatientRefundId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/patient-refunds/v1/{jsonable_encoder(patient_refund_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return HttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        patient_external_id: typing.Optional[PatientExternalId] = None,
        claim_id: typing.Optional[ClaimId] = None,
        service_line_id: typing.Optional[ServiceLineId] = None,
        billing_provider_id: typing.Optional[ProviderId] = None,
        unattributed: typing.Optional[bool] = None,
        invoice_id: typing.Optional[InvoiceId] = None,
        sources: typing.Optional[
            typing.Union[PatientTransactionSource, typing.Sequence[PatientTransactionSource]]
        ] = None,
        sort: typing.Optional[PatientRefundSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PatientRefundsPage]:
        """
        Returns all patient refunds satisfying the search criteria AND whose organization_id matches
        the current organization_id of the authenticated user.

        Parameters
        ----------
        limit : typing.Optional[int]
            Defaults to 100. The value must be greater than 0 and less than 1000.

        patient_external_id : typing.Optional[PatientExternalId]

        claim_id : typing.Optional[ClaimId]

        service_line_id : typing.Optional[ServiceLineId]

        billing_provider_id : typing.Optional[ProviderId]

        unattributed : typing.Optional[bool]
            returns payments with unattributed allocations if set to true

        invoice_id : typing.Optional[InvoiceId]

        sources : typing.Optional[typing.Union[PatientTransactionSource, typing.Sequence[PatientTransactionSource]]]

        sort : typing.Optional[PatientRefundSortField]
            Defaults to refund_timestamp

        sort_direction : typing.Optional[SortDirection]
            Sort direction. Defaults to descending order if not provided.

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PatientRefundsPage]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/patient-refunds/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "limit": limit,
                "patient_external_id": patient_external_id,
                "claim_id": claim_id,
                "service_line_id": service_line_id,
                "billing_provider_id": billing_provider_id,
                "unattributed": unattributed,
                "invoice_id": invoice_id,
                "sources": sources,
                "sort": sort,
                "sort_direction": sort_direction,
                "page_token": page_token,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PatientRefundsPage,
                parse_obj_as(
                    type_=PatientRefundsPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, patient_refund_id: PatientRefundId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PatientRefund]:
        """
        Retrieves a previously created patient refund by its `patient_refund_id`.

        Parameters
        ----------
        patient_refund_id : PatientRefundId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PatientRefund]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/patient-refunds/v1/{jsonable_encoder(patient_refund_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PatientRefund,
                parse_obj_as(
                    type_=PatientRefund,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        amount_cents: int,
        patient_external_id: PatientExternalId,
        allocations: typing.Sequence[AllocationCreate],
        refund_timestamp: typing.Optional[dt.datetime] = OMIT,
        refund_note: typing.Optional[str] = OMIT,
        invoice: typing.Optional[InvoiceId] = OMIT,
        refund_reason: typing.Optional[RefundReason] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PatientRefund]:
        """
        Creates a new patient refund record and returns the newly created PatientRefund object.
        The allocations can describe whether the refund is being applied toward a specific service line,
        claim, or billing provider.

        Parameters
        ----------
        amount_cents : int

        patient_external_id : PatientExternalId

        allocations : typing.Sequence[AllocationCreate]

        refund_timestamp : typing.Optional[dt.datetime]

        refund_note : typing.Optional[str]

        invoice : typing.Optional[InvoiceId]

        refund_reason : typing.Optional[RefundReason]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PatientRefund]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/patient-refunds/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json={
                "amount_cents": amount_cents,
                "refund_timestamp": refund_timestamp,
                "refund_note": refund_note,
                "patient_external_id": patient_external_id,
                "allocations": allocations,
                "invoice": invoice,
                "refund_reason": refund_reason,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PatientRefund,
                parse_obj_as(
                    type_=PatientRefund,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        patient_refund_id: PatientRefundId,
        *,
        refund_timestamp: typing.Optional[dt.datetime] = OMIT,
        refund_note: typing.Optional[NoteUpdate] = OMIT,
        invoice: typing.Optional[InvoiceUpdate] = OMIT,
        refund_reason: typing.Optional[RefundReasonUpdate] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PatientRefund]:
        """
        Updates the patient refund record matching the provided patient_refund_id.

        Parameters
        ----------
        patient_refund_id : PatientRefundId

        refund_timestamp : typing.Optional[dt.datetime]

        refund_note : typing.Optional[NoteUpdate]

        invoice : typing.Optional[InvoiceUpdate]

        refund_reason : typing.Optional[RefundReasonUpdate]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PatientRefund]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/patient-refunds/v1/{jsonable_encoder(patient_refund_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json={
                "refund_timestamp": refund_timestamp,
                "refund_note": refund_note,
                "invoice": invoice,
                "refund_reason": refund_reason,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PatientRefund,
                parse_obj_as(
                    type_=PatientRefund,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, patient_refund_id: PatientRefundId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Deletes the patient refund record matching the provided patient_refund_id.

        Parameters
        ----------
        patient_refund_id : PatientRefundId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/patient-refunds/v1/{jsonable_encoder(patient_refund_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return AsyncHttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
from .....commons.types.resource_page import ResourcePage
from .encounter import Encounter


class EncounterPage(ResourcePage):
    """
    Examples
    --------
    import datetime
    import uuid

    from candid.resources.billing_notes.resources.v_2 import BillingNote
    from candid.resources.claims import Claim, ClaimStatus
    from candid.resources.commons import (
        DateRangeOptionalEnd,
        FacilityTypeCode,
        InsuranceTypeCode,
        NextResponsibleParty,
        PatientRelationshipToInsuredCodeAll,
        PhoneNumber,
        PhoneNumberType,
        ProcedureModifier,
        ServiceLineUnits,
        SourceOfPaymentCode,
        State,
        StreetAddressLongZip,
        StreetAddressShortZip,
    )
    from candid.resources.custom_schemas.resources.v_1 import SchemaInstance
    from candid.resources.diagnoses import Diagnosis, DiagnosisTypeCode
    from candid.resources.encounter_providers.resources.v_2 import EncounterProvider
    from candid.resources.encounters.resources.v_4 import (
        BillableStatusType,
        ClinicalNote,
        ClinicalNoteCategory,
        CodingAttributionType,
        Encounter,
        EncounterOwnerOfNextActionType,
        EncounterPage,
        EncounterSubmissionOriginType,
        IntakeFollowUp,
        IntakeQuestion,
        IntakeResponseAndFollowUps,
        Intervention,
        InterventionCategory,
        Lab,
        LabCodeType,
        Medication,
        NoteCategory,
        PatientHistoryCategory,
        PatientHistoryCategoryEnum,
        ResponsiblePartyType,
        ServiceAuthorizationExceptionCode,
        SynchronicityType,
    )
    from candid.resources.eras import Era
    from candid.resources.guarantor.resources.v_1 import Guarantor
    from candid.resources.individual import (
        Gender,
        Patient,
        PatientNonInsurancePayerInfo,
        Subscriber,
    )
    from candid.resources.insurance_cards.resources.v_2 import InsuranceCard
    from candid.resources.invoices import Invoice, InvoiceItem, InvoiceStatus
    from candid.resources.non_insurance_payers.resources.v_1 import (
        NonInsurancePayer,
    )
    from candid.resources.patient_payments.resources.v_3 import (
        PatientPayment,
        PatientPaymentSource,
        PatientPaymentStatus,
    )
    from candid.resources.service_facility import EncounterServiceFacility
    from candid.resources.service_lines.resources.v_2 import (
        DenialReasonContent,
        ServiceLine,
        ServiceLineAdjustment,
        ServiceLineDenialReason,
        ServiceLineEraData,
        TestResult,
        TestResultType,
    )
    from candid.resources.tags import Tag, TagColorEnum

    EncounterPage(
        items=[
            Encounter(
                encounter_id=uuid.UUID(
                    "b2506e84-4470-4cba-8a68-6883367739e1",
                ),
                claims=[
                    Claim(
                        claim_id=uuid.UUID(
                            "dd9d7f82-37b5-449d-aa63-26925398335b",
                        ),
                        status=ClaimStatus.BILLER_RECEIVED,
                        clearinghouse="Change Healthcare",
                        clearinghouse_claim_id="5BA7C3AB-2BC2-496C-8B10-6CAC73D0729D",
                        payer_claim_id="9BB9F259-9756-4F16-8F53-9DBB9F7EB1BB",
                        service_lines=[
                            ServiceLine(
                                created_at=datetime.datetime.fromisoformat(
                                    "2023-01-01 00:00:00+00:00",
                                ),
                                modifiers=[ProcedureModifier.TWENTY_TWO],
                                charge_amount_cents=10000,
                                allowed_amount_cents=8000,
                                insurance_balance_cents=0,
                                patient_balance_cents=2000,
                                paid_amount_cents=8000,
                                patient_responsibility_cents=2000,
                                diagnosis_id_zero=uuid.UUID(
                                    "4ac84bcd-12f5-4f86-a57b-e06749127c98",
                                ),
                                diagnosis_id_one=uuid.UUID(
                                    "eea5ca5a-8b43-45fd-8cbd-c6cc1103e759",
                                ),
                                diagnosis_id_two=uuid.UUID(
                                    "5c4aa029-2db9-4202-916e-e93c708f65ff",
                                ),
                                diagnosis_id_three=uuid.UUID(
                                    "81795126-a3ac-443c-b47e-7259a16ab4a2",
                                ),
                                service_line_era_data=ServiceLineEraData(
                                    service_line_adjustments=[
                                        ServiceLineAdjustment(
                                            created_at=datetime.datetime.fromisoformat(
                                                "2023-01-01 00:00:00+00:00",
                                            ),
                                            adjustment_group_code="CO",
                                            adjustment_reason_code="CO",
                                            adjustment_amount_cents=1000,
                                            adjustment_note="test_note",
                                        )
                                    ],
                                    remittance_advice_remark_codes=["N362"],
                                ),
                                service_line_manual_adjustments=[
                                    ServiceLineAdjustment(
                                        created_at=datetime.datetime.fromisoformat(
                                            "2023-01-01 00:00:00+00:00",
                                        ),
                                        adjustment_group_code="CO",
                                        adjustment_reason_code="CO",
                                        adjustment_amount_cents=1000,
                                        adjustment_note="test_note",
                                    )
                                ],
                                related_invoices=[
                                    Invoice(
                                        id=uuid.UUID(
                                            "901be2f1-41bc-456e-9987-4fe2f84f9d75",
                                        ),
                                        created_at=datetime.datetime.fromisoformat(
                                            "2023-01-01 00:00:00+00:00",
                                        ),
                                        updated_at=datetime.datetime.fromisoformat(
                                            "2023-01-01 00:00:00+00:00",
                                        ),
                                        organzation_id=uuid.UUID(
                                            "f13f73d4-4344-46ea-9d93-33bcffbb9f36",
                                        ),
                                        source_id="9B626577-8808-4F28-9ED1-F0DFF0D49BBC",
                                        source_customer_id="624D1972-8C69-4C2F-AEFA-10856F734DB3",
                                        patient_external_id="10FED4D6-4C5A-48DF-838A-EEF45A74788D",
                                        note="test_note",
                                        due_date="2023-10-10",
                                        status=InvoiceStatus.DRAFT,
                                        url="https://example.com",
                                        customer_invoice_url="https://example.com",
                                        items=[
                                            InvoiceItem(
                                                service_line_id=uuid.UUID(
                                                    "ced00f23-6e68-4678-9dbc-f5aa2969a565",
                                                ),
                                                amount_cents=500,
                                            )
                                        ],
                                    )
                                ],
                                denial_reason=ServiceLineDenialReason(
                                    reason=DenialReasonContent.AUTHORIZATION_REQUIRED,
                                ),
                                place_of_service_code=FacilityTypeCode.PHARMACY,
                                service_line_id=uuid.UUID(
                                    "ced00f23-6e68-4678-9dbc-f5aa2969a565",
                                ),
                                procedure_code="99213",
                                quantity="1",
                                units=ServiceLineUnits.MJ,
                                claim_id=uuid.UUID(
                                    "026a1fb8-748e-4859-a2d7-3ea9e07d25ae",
                                ),
                                date_of_service_range=DateRangeOptionalEnd(
                                    start_date="2023-01-01",
                                    end_date="2023-01-03",
                                ),
                                date_of_service=datetime.date.fromisoformat(
                                    "2023-01-01",
                                ),
                                end_date_of_service=datetime.date.fromisoformat(
                                    "2023-01-03",
                                ),
                                test_results=[
                                    TestResult(
                                        result_type=TestResultType.HEMOGLOBIN,
                                        value=51.0,
                                    ),
                                    TestResult(
                                        result_type=TestResultType.LDL,
                                        value=110.0,
                                    ),
                                ],
                            )
                        ],
                        eras=[
                            Era(
                                era_id=uuid.UUID(
                                    "4d844ef1-2253-43cd-a4f1-6db7e65cb54b",
                                ),
                                check_number="CHK12345",
                                check_date="2023-10-12",
                            )
                        ],
                    )
                ],
                patient=Patient(
                    individual_id=uuid.UUID(
                        "93ddbebf-4956-4482-9a6c-21499b7e4e5d",
                    ),
                    phone_numbers=[
                        PhoneNumber(
                            number="1234567890",
                            type=PhoneNumberType.HOME,
                        )
                    ],
                    non_insurance_payers=[
                        NonInsurancePayer(
                            non_insurance_payer_id=uuid.UUID(
                                "eb7623ab-d5bc-4b25-b257-2b8fcec578de",
                            ),
                            name="Sunrise Foundation",
                            category="Foundation",
                            description="Sunrise Foundation is a non-profit organization that provides financial assistance to patients in need.",
                            enabled=True,
                            address=StreetAddressShortZip(
                                address_1="123 Main St",
                                city="San Francisco",
                                state=State.CA,
                                zip_code="94105",
                            ),
                            clinical_trials=[],
                        )
                    ],
                    non_insurance_payers_info=[
                        PatientNonInsurancePayerInfo(
                            non_insurance_payer=NonInsurancePayer(
                                non_insurance_payer_id=uuid.UUID(
                                    "eb7623ab-d5bc-4b25-b257-2b8fcec578de",
                                ),
                                name="Sunrise Foundation",
                                category="Foundation",
                                description="Sunrise Foundation is a non-profit organization that provides financial assistance to patients in need.",
                                enabled=True,
                                address=StreetAddressShortZip(
                                    address_1="123 Main St",
                                    city="San Francisco",
                                    state=State.CA,
                                    zip_code="94105",
                                ),
                                clinical_trials=[],
                            ),
                            member_id="123456789",
                            clinical_trial_info=[],
                        )
                    ],
                    phone_consent=True,
                    email="johndoe@joincandidhealth.com",
                    email_consent=True,
                    auto_charge_consent=True,
                    external_id="49460F77-6456-41F1-AC6D-0AED08614D39",
                    date_of_birth=datetime.date.fromisoformat(
                        "2000-01-01",
                    ),
                    address=StreetAddressShortZip(
                        address_1="123 Main St",
                        address_2="Apt 1",
                        city="New York",
                        state=State.NY,
                        zip_code="10001",
                        zip_plus_four_code="1234",
                    ),
                    first_name="John",
                    last_name="Doe",
                    gender=Gender.MALE,
                ),
                guarantor=Guarantor(
                    guarantor_id=uuid.UUID(
                        "8bbdbe63-58d3-4d40-98c9-40403c050977",
                    ),
                    phone_numbers=[
                        PhoneNumber(
                            number="1234567890",
                            type=PhoneNumberType.HOME,
                        )
                    ],
                    phone_consent=True,
                    email="johndoe@joincandidhealth.com",
                    email_consent=True,
                    auto_charge_consent=True,
                    first_name="John",
                    last_name="Doe",
                    external_id="49460F77-6456-41F1-AC6D-0AED08614D39",
                    date_of_birth=datetime.date.fromisoformat(
                        "2000-01-01",
                    ),
                    address=StreetAddressShortZip(
                        address_1="123 Main St",
                        address_2="Apt 1",
                        city="New York",
                        state=State.NY,
                        zip_code="10001",
                        zip_plus_four_code="1234",
                    ),
                ),
                billing_provider=EncounterProvider(
                    provider_id=uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                    address=StreetAddressLongZip(
                        address_1="123 Main St",
                        address_2="Apt 1",
                        city="New York",
                        state=State.NY,
                        zip_code="10001",
                        zip_plus_four_code="1234",
                    ),
                    tax_id="123456789",
                    npi="1234567890",
                    taxonomy_code="207Q00000X",
                    first_name="John",
                    last_name="Doe",
                    organization_name="Organization Name",
                ),
                rendering_provider=EncounterProvider(
                    provider_id=uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                    address=StreetAddressLongZip(
                        address_1="123 Main St",
                        address_2="Apt 1",
                        city="New York",
                        state=State.NY,
                        zip_code="10001",
                        zip_plus_four_code="1234",
                    ),
                    tax_id="123456789",
                    npi="1234567890",
                    taxonomy_code="207Q00000X",
                    first_name="John",
                    last_name="Doe",
                    organization_name="Organization Name",
                ),
                referring_provider=EncounterProvider(
                    provider_id=uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                    address=StreetAddressLongZip(
                        address_1="123 Main St",
                        address_2="Apt 1",
                        city="New York",
                        state=State.NY,
                        zip_code="10001",
                        zip_plus_four_code="1234",
                    ),
                    tax_id="123456789",
                    npi="1234567890",
                    taxonomy_code="207Q00000X",
                    first_name="John",
                    last_name="Doe",
                    organization_name="Organization Name",
                ),
                service_facility=EncounterServiceFacility(
                    service_facility_id=uuid.UUID(
                        "2861487b-232c-4ded-a874-616a5db0688f",
                    ),
                    organization_name="Test Organization",
                    address=StreetAddressLongZip(
                        address_1="123 Main St",
                        address_2="Apt 1",
                        city="New York",
                        state=State.NY,
                        zip_code="10001",
                        zip_plus_four_code="1234",
                    ),
                ),
                subscriber_primary=Subscriber(
                    individual_id=uuid.UUID(
                        "797348a9-e7e8-4e59-8628-95390d079c0b",
                    ),
                    insurance_card=InsuranceCard(
                        insurance_card_id=uuid.UUID(
                            "ca5b7711-4419-4161-9b7c-3494ac40c8d4",
                        ),
                        member_id="E85313B4-0FFC-4119-8042-8161A4ECFF0A",
                        payer_name="John Doe",
                        payer_id="836DDAA6-863F-4020-ACCA-205A689F0002",
                        rx_bin="610014",
                        rx_pcn="MEDDPRIME",
                        image_url_front="https://s3.amazonaws.com/front.jpg",
                        image_url_back="https://s3.amazonaws.com/back.jpg",
                        group_number="ABC12345",
                        plan_name="Silver PPO Plan",
                        plan_type=SourceOfPaymentCode.SELF_PAY,
                        insurance_type=InsuranceTypeCode.C_12,
                    ),
                    patient_relationship_to_subscriber_code=PatientRelationshipToInsuredCodeAll.SPOUSE,
                    date_of_birth=datetime.date.fromisoformat(
                        "2000-01-01",
                    ),
                    address=StreetAddressShortZip(
                        address_1="123 Main St",
                        address_2="Apt 1",
                        city="New York",
                        state=State.NY,
                        zip_code="10001",
                        zip_plus_four_code="1234",
                    ),
                    first_name="John",
                    last_name="Doe",
                    gender=Gender.MALE,
                ),
                subscriber_secondary=Subscriber(
                    individual_id=uuid.UUID(
                        "797348a9-e7e8-4e59-8628-95390d079c0b",
                    ),
                    insurance_card=InsuranceCard(
                        insurance_card_id=uuid.UUID(
                            "ca5b7711-4419-4161-9b7c-3494ac40c8d4",
                        ),
                        member_id="E85313B4-0FFC-4119-8042-8161A4ECFF0A",
                        payer_name="John Doe",
                        payer_id="836DDAA6-863F-4020-ACCA-205A689F0002",
                        rx_bin="610014",
                        rx_pcn="MEDDPRIME",
                        image_url_front="https://s3.amazonaws.com/front.jpg",
                        image_url_back="https://s3.amazonaws.com/back.jpg",
                        group_number="ABC12345",
                        plan_name="Silver PPO Plan",
                        plan_type=SourceOfPaymentCode.SELF_PAY,
                        insurance_type=InsuranceTypeCode.C_12,
                    ),
                    patient_relationship_to_subscriber_code=PatientRelationshipToInsuredCodeAll.SPOUSE,
                    date_of_birth=datetime.date.fromisoformat(
                        "2000-01-01",
                    ),
                    address=StreetAddressShortZip(
                        address_1="123 Main St",
                        address_2="Apt 1",
                        city="New York",
                        state=State.NY,
                        zip_code="10001",
                        zip_plus_four_code="1234",
                    ),
                    first_name="John",
                    last_name="Doe",
                    gender=Gender.MALE,
                ),
                url="https://example.com",
                diagnoses=[
                    Diagnosis(
                        diagnosis_id=uuid.UUID(
                            "5c770e00-4bbf-42af-a73f-99c5e91fc0db",
                        ),
                        created_at=datetime.datetime.fromisoformat(
                            "2023-01-01 00:00:00+00:00",
                        ),
                        updated_at=datetime.datetime.fromisoformat(
                            "2023-01-01 00:00:00+00:00",
                        ),
                        encounter_id=uuid.UUID(
                            "3f63985b-51a4-4dd4-9418-7d50b2520792",
                        ),
                        name="John Doe",
                        code_type=DiagnosisTypeCode.ABF,
                        code="I10",
                    )
                ],
                clinical_notes=[
                    ClinicalNoteCategory(
                        category=NoteCategory.CLINICAL,
                        notes=["Patient complained of mild chest pain."],
                        notes_structured=[
                            ClinicalNote(
                                text="Mild chest pain since morning.",
                                author_name="John Doe",
                                author_npi="1234567890",
                                timestamp=datetime.datetime.fromisoformat(
                                    "2023-01-01 00:00:00+00:00",
                                ),
                            )
                        ],
                    )
                ],
                billing_notes=[
                    BillingNote(
                        billing_note_id=uuid.UUID(
                            "99882eea-936f-4e71-bc4f-520e4d14e3e2",
                        ),
                        encounter_id=uuid.UUID(
                            "8bcfb6a8-2876-4111-9e3f-602b541fcf62",
                        ),
                        created_at=datetime.datetime.fromisoformat(
                            "2023-01-01 00:00:00+00:00",
                        ),
                        author_auth_0_id="F0DE3BF9-F9A1-4FA7-BF6B-28C0B46BADD8",
                        author_name="John Doe",
                        text="Patient was billed for an MRI.",
                    )
                ],
                place_of_service_code=FacilityTypeCode.PHARMACY,
                place_of_service_code_as_submitted=FacilityTypeCode.PHARMACY,
                patient_histories=[
                    PatientHistoryCategory(
                        category=PatientHistoryCategoryEnum.PRESENT_ILLNESS,
                        questions=[
                            IntakeQuestion(
                                id="6E7FBCE4-A8EA-46D0-A8D8-FF83CA3BB176",
                                text="Do you have any allergies?",
                                responses=[
                                    IntakeResponseAndFollowUps(
                                        response="No allergies",
                                        follow_ups=[
                                            IntakeFollowUp(
                                                id="4F3D57F9-AC94-49D6-87E4-E804B709917A",
                                                text="Do you have any allergies?",
                                                response="No allergies",
                                            )
                                        ],
                                    )
                                ],
                            )
                        ],
                    )
                ],
                patient_payments=[
                    PatientPayment(
                        patient_payment_id="CF237BE1-E793-4BBF-8958-61D5179D1D0D",
                        organization_id=uuid.UUID(
                            "0788ca2a-b20d-4b8e-b8d4-07fa0b3b4907",
                        ),
                        source_internal_id="D1A76039-D5C5-4323-A2FC-B7C8B6AEF6A4",
                        source=PatientPaymentSource.MANUAL_ENTRY,
                        amount_cents=2000,
                        payment_timestamp=datetime.datetime.fromisoformat(
                            "2023-01-01 00:00:00+00:00",
                        ),
                        status=PatientPaymentStatus.PENDING,
                        payment_name="John Doe",
                        payment_note="test payment note",
                        patient_external_id="B7437260-D6B4-48CF-B9D7-753C09F34E76",
                        encounter_external_id="0F26B9C3-199F-4CBB-A166-B87EA7C631BB",
                        service_line_id=uuid.UUID(
                            "b557dc86-c629-478c-850a-02d45ac11783",
                        ),
                    )
                ],
                tags=[
                    Tag(
                        creator_id="00EB5A46-35C6-441B-9751-AF307AEF5888",
                        tag_id="void-claim-submitted",
                        description="to indicate claims where a void claim has been submitted",
                        color=TagColorEnum.BLACK,
                    )
                ],
                coding_attribution=CodingAttributionType.CANDID,
                work_queue_id="12",
                work_queue_membership_activated_at=datetime.datetime.fromisoformat(
                    "2023-01-01 00:00:00+00:00",
                ),
                owner_of_next_action=EncounterOwnerOfNextActionType.CANDID,
                submission_origin=EncounterSubmissionOriginType.CANDID,
                external_id="5C21490F-A9C0-45F4-B5DB-136E3AEC617A",
                date_of_service=datetime.date.fromisoformat(
                    "2023-01-01",
                ),
                end_date_of_service=datetime.date.fromisoformat(
                    "2023-01-05",
                ),
                prior_authorization_number="PA1234567",
                patient_authorized_release=True,
                benefits_assigned_to_provider=True,
                provider_accepts_assignment=True,
                appointment_type="Routine Checkup",
                existing_medications=[
                    Medication(
                        name="Lisinopril",
                        rx_cui="860975",
                        dosage="10mg",
                        dosage_form="Tablet",
                        frequency="Once Daily",
                        as_needed=True,
                    )
                ],
                interventions=[
                    Intervention(
                        name="Physical Therapy Session",
                        category=InterventionCategory.LIFESTYLE,
                        description="A session focused on improving muscular strength, flexibility, and range of motion post-injury.",
                        medication=Medication(
                            name="Lisinopril",
                            rx_cui="860975",
                            dosage="10mg",
                            dosage_form="Tablet",
                            frequency="Once Daily",
                            as_needed=True,
                        ),
                        labs=[
                            Lab(
                                name="Genetic Health Labs",
                                code="GH12345",
                                code_type=LabCodeType.QUEST,
                            )
                        ],
                    )
                ],
                pay_to_address=StreetAddressLongZip(
                    address_1="123 Main St",
                    address_2="Apt 1",
                    city="New York",
                    state=State.NY,
                    zip_code="10001",
                    zip_plus_four_code="1234",
                ),
                synchronicity=SynchronicityType.SYNCHRONOUS,
                billable_status=BillableStatusType.BILLABLE,
                responsible_party=ResponsiblePartyType.INSURANCE_PAY,
                service_authorization_exception_code=ServiceAuthorizationExceptionCode.C_1,
                admission_date=datetime.date.fromisoformat(
                    "2023-01-01",
                ),
                discharge_date=datetime.date.fromisoformat(
                    "2023-01-05",
                ),
                onset_of_current_illness_or_symptom_date=datetime.date.fromisoformat(
                    "2023-01-01",
                ),
                schema_instances=[
                    SchemaInstance(
                        schema_id=uuid.UUID(
                            "ec096b13-f80a-471d-aaeb-54b021c9d582",
                        ),
                        content={
                            "provider_category": "internist",
                            "is_urgent_care": True,
                            "bmi": 24.2,
                            "age": 38,
                        },
                    )
                ],
                created_at=datetime.datetime.fromisoformat(
                    "2023-01-01 00:00:00+00:00",
                ),
                next_responsible_party=NextResponsibleParty.PRIMARY,
            )
        ],
    )
    """

    items: typing.List[Encounter]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core import enum

T_Result = typing.TypeVar("T_Result")


class Primitive(enum.StrEnum):
    """
    The BOOLEAN and STRING primitives respectively map to the `boolean` and `string` JSON data types.
    The DOUBLE and INTEGER primitives must be written as a JSON `number` type.
    """

    BOOLEAN = "BOOLEAN"
    DOUBLE = "DOUBLE"
    INTEGER = "INTEGER"
    STRING = "STRING"
    _UNKNOWN = "__PRIMITIVE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "Primitive":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        boolean: typing.Callable[[], T_Result],
        double: typing.Callable[[], T_Result],
        integer: typing.Callable[[], T_Result],
        string: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is Primitive.BOOLEAN:
            return boolean()
        if self is Primitive.DOUBLE:
            return double()
        if self is Primitive.INTEGER:
            return integer()
        if self is Primitive.STRING:
            return string()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...commons.types.appointment_id import AppointmentId
from ...commons.types.claim_id import ClaimId
from ...commons.types.encounter_external_id import EncounterExternalId
from ...commons.types.patient_external_id import PatientExternalId
from ...commons.types.provider_id import ProviderId
from ...commons.types.service_line_id import ServiceLineId


class AllocationTargetCreate_ServiceLineById(UniversalBaseModel):
    value: ServiceLineId
    type: typing.Literal["service_line_by_id"] = "service_line_by_id"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class AllocationTargetCreate_ClaimById(UniversalBaseModel):
    value: ClaimId
    type: typing.Literal["claim_by_id"] = "claim_by_id"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class AllocationTargetCreate_ClaimByEncounterExternalId(UniversalBaseModel):
    value: EncounterExternalId
    type: typing.Literal["claim_by_encounter_external_id"] = "claim_by_encounter_external_id"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class AllocationTargetCreate_BillingProviderById(UniversalBaseModel):
    value: ProviderId
    type: typing.Literal["billing_provider_by_id"] = "billing_provider_by_id"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class AllocationTargetCreate_AppointmentByIdAndPatientExternalId(UniversalBaseModel):
    """
    Allocation targets describe whether the portion of a payment is being applied toward a specific service line,
    claim, billing provider, or is unallocated.
    """

    type: typing.Literal["appointment_by_id_and_patient_external_id"] = "appointment_by_id_and_patient_external_id"
    appointment_id: AppointmentId
    patient_external_id: PatientExternalId

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class AllocationTargetCreate_Unattributed(UniversalBaseModel):
    """
    Allocation targets describe whether the portion of a payment is being applied toward a specific service line,
    claim, billing provider, or is unallocated.
    """

    type: typing.Literal["unattributed"] = "unattributed"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


AllocationTargetCreate = typing_extensions.Annotated[
    typing.Union[
        AllocationTargetCreate_ServiceLineById,
        AllocationTargetCreate_ClaimById,
        AllocationTargetCreate_ClaimByEncounterExternalId,
        AllocationTargetCreate_BillingProviderById,
        AllocationTargetCreate_AppointmentByIdAndPatientExternalId,
        AllocationTargetCreate_Unattributed,
    ],
    pydantic.Field(discriminator="type"),
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class IdpUserMetadata_AuthZeroMetadata(UniversalBaseModel):
    """
    Examples
    --------
    from candid.resources.users.resources.v_2 import (
        IdpUserMetadata_AuthZeroMetadata,
    )

    IdpUserMetadata_AuthZeroMetadata(
        auth_0_id="auth0|5f9b3b5a7b0b0d006a2b0b0b",
    )
    """

    type: typing.Literal["auth_zero_metadata"] = "auth_zero_metadata"
    auth_0_id: str = pydantic.Field(alias="auth0_id")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class IdpUserMetadata_GoogleAppsMetadata(UniversalBaseModel):
    """
    Examples
    --------
    from candid.resources.users.resources.v_2 import (
        IdpUserMetadata_AuthZeroMetadata,
    )

    IdpUserMetadata_AuthZeroMetadata(
        auth_0_id="auth0|5f9b3b5a7b0b0d006a2b0b0b",
    )
    """

    type: typing.Literal["google_apps_metadata"] = "google_apps_metadata"
    google_apps_id: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class IdpUserMetadata_OtherIdpMetadata(UniversalBaseModel):
    """
    Examples
    --------
    from candid.resources.users.resources.v_2 import (
        IdpUserMetadata_AuthZeroMetadata,
    )

    IdpUserMetadata_AuthZeroMetadata(
        auth_0_id="auth0|5f9b3b5a7b0b0d006a2b0b0b",
    )
    """

    type: typing.Literal["other_idp_metadata"] = "other_idp_metadata"
    idp_external_id: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


"""
from candid.resources.users.resources.v_2 import (
    IdpUserMetadata_AuthZeroMetadata,
)

IdpUserMetadata_AuthZeroMetadata(
    auth_0_id="auth0|5f9b3b5a7b0b0d006a2b0b0b",
)
"""
IdpUserMetadata = typing_extensions.Annotated[
    typing.Union[
        IdpUserMetadata_AuthZeroMetadata, IdpUserMetadata_GoogleAppsMetadata, IdpUserMetadata_OtherIdpMetadata
    ],
    pydantic.Field(discriminator="type"),
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .intervention_category import InterventionCategory
from .lab import Lab
from .medication import Medication


class Intervention(UniversalBaseModel):
    """
    Examples
    --------
    from candid.resources.encounters.resources.v_4 import (
        Intervention,
        InterventionCategory,
        Lab,
        LabCodeType,
        Medication,
    )

    Intervention(
        name="Physical Therapy Session",
        category=InterventionCategory.LIFESTYLE,
        description="A session focused on improving muscular strength, flexibility, and range of motion post-injury.",
        medication=Medication(
            name="Lisinopril",
            rx_cui="860975",
            dosage="10mg",
            dosage_form="Tablet",
            frequency="Once Daily",
            as_needed=True,
        ),
        labs=[
            Lab(
                name="Genetic Health Labs",
                code="GH12345",
                code_type=LabCodeType.QUEST,
            )
        ],
    )
    """

    name: str
    category: InterventionCategory
    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    "Examples: 'Birth Control LAC', 'Tracking', 'Stress Management', 'Supplement', 'Labs'"
    """

    medication: typing.Optional[Medication] = pydantic.Field(default=None)
    """
    Required when `type` is `allopathic`.
    """

    labs: typing.Optional[typing.List[Lab]] = pydantic.Field(default=None)
    """
    Required when `type` is `tests`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ........core import enum

T_Result = typing.TypeVar("T_Result")


class CoverageValueUnit(enum.StrEnum):
    PERCENT = "PERCENT"
    CURRENCY = "CURRENCY"
    COUNT = "COUNT"
    _UNKNOWN = "__COVERAGEVALUEUNIT_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "CoverageValueUnit":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        percent: typing.Callable[[], T_Result],
        currency: typing.Callable[[], T_Result],
        count: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is CoverageValueUnit.PERCENT:
            return percent()
        if self is CoverageValueUnit.CURRENCY:
            return currency()
        if self is CoverageValueUnit.COUNT:
            return count()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class TypeOfBillFrequencyCode(enum.StrEnum):
    C_0 = "0"
    """
    NON-PAYMENT/ZERO CLAIM - Use this code when the bill is submitted to a payer for informational purposes, the provider does not anticipate payment to result from submitting the bill; but needs to inform the payer of the non-reimbursable periods of confinement or termination of care.
    """

    C_1 = "1"
    """
    ADMIT THROUGH DISCHARGE CLAIM (A) - Use this code when billing for a confined treatment or inpatient period. This will include bills representing a total confinement or course of treatment, and bills that represent an entire benefit period of the primary third party payer. Note: Use Occurrence Code 42 to indicate the date of discharge when the "Through" date in Form Locator 06 (Statement Covers Period) is not the actual discharge date and the frequency code in Form Locator 04 is that of a final bill, i.e., 1, 4, and 7 (when the replacement is for a prior final claim).
    """

    C_2 = "2"
    """
    INTERIM - FIRST CLAIM - This code is to be used for the first of a series of bills to the same third party payer for the same confinement or course of treatment.
    """

    C_3 = "3"
    """
    INTERIM - CONTINUING CLAIM (B) - This code is to be used when a bill for the same confinement or course of treatment has previously been submitted and it is expected that further bills for the same confinement or course of treatment will be submitted.
    """

    C_4 = "4"
    """
    INTERIM LAST CLAIM (B) - Use this code for the last of a series of bills, for which payment is expected, to the same third party payer for the same confinement or course of treatment. Note: Do not use this code in lieu of a code for Late Charges or Non-Payment/Zero Claims.
    """

    C_5 = "5"
    """
    LATE CHARGE(S) ONLY - Use this code for submitting charges to the payer, which were received by the provider after the Admit Through Discharge, or the Last Interim Claim has been submitted. Note: This code is not intended to be used in lieu of a Replacement Claim.
    """

    C_7 = "7"
    """
    REPLACEMENT OF PRIOR CLAIM (A) - This code is used when a specific bill has been issued for a specific Provider, Patient, Payer, Insured and "Statement Covers Period" and it needs to be restated in its entirety, except for the same identity information. In using this code, the payer is to operate on the principle to void the original bill, and that the information present on this bill represents a complete replacement of the previously issued bill. However, this code is not intended to be used in lieu of a Late Charge(s) Only claim.
    """

    C_8 = "8"
    """
    VOID/CANCEL OF PRIOR CLAIM (A) - This code reflects the elimination in its entirety of a previously submitted bill for a specific Provider, Patient, Payer, Insured and "Statement Covers Period." The provider may wish to follow a Void Bill with a bill containing the correct information when a Payer is unable to process a Replacement to a Prior Claim. The appropriate Frequency Code must be used when submitting the new bill.
    """

    C_9 = "9"
    """
    FINAL CLAIM FOR A HOME HEALTH PPS EPISODE (EFFECTIVE FOR SERVICE DATES PRIOR TO 12/1/21 TO BE DISCONTINUED 1/1/22) FINAL CLAIM FOR A HOME HEALTH PPS PERIOD (EFFECTIVE 1/1/22) - This code indicates the HH bill should be processed as a debit or credit adjustment to the initial home health PPS bill. This code is specific to home health and does not replace Frequency Codes 7 or 8. For claims with service dates on or after January 1, 2023, the HH bill is processed as an original claim, not an adjustment of an initial bill. This code indicates a HH original bill to be processed following the submission of an HH PPS Notice of Admission (NOA-TOB 32A).
    """

    CA = "A"
    """
    ADMISSION/ELECTION NOTICE - This code is used when a hospice, home health agency, CMS Coordinated Care Demonstration entity, Centers of Excellence Demonstration entity, Provider Partnerships Demonstration entity or Religious Non-medical Health Care Institution is submitting the UB-04 as an admission or election notice.
    """

    CB = "B"
    """
    TERMINATION/REVOCATION NOTICE - This code is used when the UB-04 is used as a Termination/Revocation of a hospice, Medicare Coordinated Care Demonstration, Centers of Excellence Demonstration, Provider Partnerships Demonstration or Religious Non-Medical Care Institution election.
    """

    CC = "C"
    """
    HOSPICE CHANGE OF PROVIDER NOTICE - Use when the UB-04 is used as a Notice of Change to the hospice provider.
    """

    CD = "D"
    """
    CANCELLATION OF ELECTION NOTICE - This code is used when the UB-04 is used as a Notice of a Void/Cancel of a hospice, home health agency, CMS Coordinated Care Demonstration entity, Centers of Excellence Demonstration entity, Provider Partnerships Demonstration entity or Religious Non-medical Health Care Institution election.
    """

    CE = "E"
    """
    HOSPICE CHANGE OF OWNERSHIP - This code is used to indicate a Notice of Change in Ownership for the hospice.
    """

    CF = "F"
    """
    BENEFICIARY INITIATED ADJUSTMENT CLAIM - For intermediary use only, use to identify adjustments initiated by the beneficiary.
    """

    CG = "G"
    """
    CWF INITIATED ADJUSTMENT CLAIM - For intermediary use only, to identify adjustments initiated by CWF.
    """

    CH = "H"
    """
    CMS INITIATED ADJUSTMENT - For intermediary use only, to identify adjustments initiated by CMS.
    """

    CI = "I"
    """
    INTERMEDIARY ADJUSTMENT CLAIM (OTHER THAN QIO OR PROVIDER) - For intermediary use only, to identify adjustments initiated by the intermediary.
    """

    CJ = "J"
    """
    INITIATED ADJUSTMENT CLAIM - OTHER - For intermediary use only, to identify adjustments initiated by other entities.
    """

    CK = "K"
    """
    OIG INITIATED ADJUSTMENT CLAIM - For intermediary use only, to identify adjustments initiated by OIG.
    """

    CM = "M"
    """
    MSP INITIATED ADJUSTMENT CLAIM - For intermediary use only, to identify adjustments initiated by MSP. Note: MSP takes precedence over other adjustment sources.
    """

    CO_FREQ = "O"
    """
    NONPAYMENT/ZERO CLAIMS - This code is used when you do not anticipate payment from the payer for the bill, but you are informing the payer about a period of non-payable confinement or termination of care. The "Through" date of this bill (FL 06) is the discharge date for this confinement. Medicare requires "nonpayment" bills only to extend the spell-of-illness in inpatient cases. Other nonpayment bills are not needed and may be returned.
    """

    CP = "P"
    """
    QIO ADJUSTMENT CLAIM - For intermediary use only, to identify an adjustment initiated because of a QIO review.
    """

    CQ = "Q"
    """
    CLAIM SUBMITTED FOR RECONSIDERATION/REOPENING OUTSIDE OF TIMELY LIMITS (EFFECTIVE JANUARY 1, 2016) - This code is used to identify claims submitted for reconsideration that fall outside of the payer's timely filing limits.
    """

    CX = "X"
    """
    VOID/CANCEL A PRIOR ABBREVIATED ENCOUNTER SUBMISSION - This code is used by a Medicare Advantage contractor or other plan required to submit encounter data that indicates that this encounter data submission is an exact duplicate of an incorrect previous encounter data submission using the abbreviated UB-04 format. A code "Y" (Replacement of Prior Abbreviated Encounter Submission) is also submitted by the plan showing corrected information.
    """

    CY_FREQ = "Y"
    """
    REPLACEMENT OF PRIOR ABBREVIATED ENCOUNTER SUBMISSION - This code is used by a Medicare Advantage contractor or other plan required to submit encounter data when it wants to correct a previous encounter submission using the abbreviated UB-04 format. This is the code applied to the corrected or new encounter.
    """

    CZ_FREQ = "Z"
    """
    NEW ABBREVIATED ENCOUNTER SUBMISSION - This code is used by a Medicare Advantage contractor or other plan required to submit encounter data to indicate it is submitting new encounter data using the abbreviated UB-04 format. It is applicable for both inpatient and outpatient services.
    """

    _UNKNOWN = "__TYPEOFBILLFREQUENCYCODE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "TypeOfBillFrequencyCode":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        c_0: typing.Callable[[], T_Result],
        c_1: typing.Callable[[], T_Result],
        c_2: typing.Callable[[], T_Result],
        c_3: typing.Callable[[], T_Result],
        c_4: typing.Callable[[], T_Result],
        c_5: typing.Callable[[], T_Result],
        c_7: typing.Callable[[], T_Result],
        c_8: typing.Callable[[], T_Result],
        c_9: typing.Callable[[], T_Result],
        ca: typing.Callable[[], T_Result],
        cb: typing.Callable[[], T_Result],
        cc: typing.Callable[[], T_Result],
        cd: typing.Callable[[], T_Result],
        ce: typing.Callable[[], T_Result],
        cf: typing.Callable[[], T_Result],
        cg: typing.Callable[[], T_Result],
        ch: typing.Callable[[], T_Result],
        ci: typing.Callable[[], T_Result],
        cj: typing.Callable[[], T_Result],
        ck: typing.Callable[[], T_Result],
        cm: typing.Callable[[], T_Result],
        co_freq: typing.Callable[[], T_Result],
        cp: typing.Callable[[], T_Result],
        cq: typing.Callable[[], T_Result],
        cx: typing.Callable[[], T_Result],
        cy_freq: typing.Callable[[], T_Result],
        cz_freq: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is TypeOfBillFrequencyCode.C_0:
            return c_0()
        if self is TypeOfBillFrequencyCode.C_1:
            return c_1()
        if self is TypeOfBillFrequencyCode.C_2:
            return c_2()
        if self is TypeOfBillFrequencyCode.C_3:
            return c_3()
        if self is TypeOfBillFrequencyCode.C_4:
            return c_4()
        if self is TypeOfBillFrequencyCode.C_5:
            return c_5()
        if self is TypeOfBillFrequencyCode.C_7:
            return c_7()
        if self is TypeOfBillFrequencyCode.C_8:
            return c_8()
        if self is TypeOfBillFrequencyCode.C_9:
            return c_9()
        if self is TypeOfBillFrequencyCode.CA:
            return ca()
        if self is TypeOfBillFrequencyCode.CB:
            return cb()
        if self is TypeOfBillFrequencyCode.CC:
            return cc()
        if self is TypeOfBillFrequencyCode.CD:
            return cd()
        if self is TypeOfBillFrequencyCode.CE:
            return ce()
        if self is TypeOfBillFrequencyCode.CF:
            return cf()
        if self is TypeOfBillFrequencyCode.CG:
            return cg()
        if self is TypeOfBillFrequencyCode.CH:
            return ch()
        if self is TypeOfBillFrequencyCode.CI:
            return ci()
        if self is TypeOfBillFrequencyCode.CJ:
            return cj()
        if self is TypeOfBillFrequencyCode.CK:
            return ck()
        if self is TypeOfBillFrequencyCode.CM:
            return cm()
        if self is TypeOfBillFrequencyCode.CO_FREQ:
            return co_freq()
        if self is TypeOfBillFrequencyCode.CP:
            return cp()
        if self is TypeOfBillFrequencyCode.CQ:
            return cq()
        if self is TypeOfBillFrequencyCode.CX:
            return cx()
        if self is TypeOfBillFrequencyCode.CY_FREQ:
            return cy_freq()
        if self is TypeOfBillFrequencyCode.CZ_FREQ:
            return cz_freq()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .organization_id import OrganizationId
from .user_id import UserId


class BaseModel(UniversalBaseModel):
    organization_id: OrganizationId = pydantic.Field()
    """
    The organization that owns this object.
    """

    deactivated: bool = pydantic.Field()
    """
    True if the object is deactivated.  Deactivated objects are not returned in search results but are returned in all other endpoints including scan.
    """

    version: int = pydantic.Field()
    """
    The version of the object. Any update to any property of an object object will create a new version.
    """

    updated_at: dt.datetime
    updating_user_id: UserId = pydantic.Field()
    """
    The user ID of the user who last updated the object.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ...insurance_cards.resources.v_2.types.insurance_card_create_optional import InsuranceCardCreateOptional
from .subscriber_base_optional import SubscriberBaseOptional


class SubscriberCreateOptional(SubscriberBaseOptional):
    insurance_card: typing.Optional[InsuranceCardCreateOptional] = pydantic.Field(default=None)
    """
    Please reference our [Payer Information](https://docs.joincandidhealth.com/introduction/payer-information) documentation for more details on how to populate the `insurance_card` fields.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core import enum

T_Result = typing.TypeVar("T_Result")


class PatientRelationshipToInsuredCodeAll(enum.StrEnum):
    SPOUSE = "01"
    """
    Spouse
    """

    GRANDPARENT = "04"
    """
    Grandparent
    """

    GRANDCHILD = "05"
    """
    Grandchild
    """

    NIECE_NEPHEW = "07"
    """
    Niece/Nephew
    """

    FOSTER_CHILD = "10"
    """
    Foster Child
    """

    WARD_OF_THE_COURT = "15"
    """
    Ward of the Court
    """

    STEPCHILD = "17"
    """
    Stepchild
    """

    SELF = "18"
    """
    Self
    """

    CHILD = "19"
    """
    Child
    """

    EMPLOYEE = "20"
    """
    Employee
    """

    UNKNOWN = "21"
    """
    Unknown
    """

    HANDICAPPED_DEPENDENT = "22"
    """
    Handicapped Dependent
    """

    SPONSORED_DEPENDENT = "23"
    """
    Sponsored Dependent
    """

    DEPENDENT_OF_MINOR_DEPENDENT = "24"
    """
    Dependent of Minor Dependent
    """

    SIGNIFICANT_OTHER = "29"
    """
    Significant Other
    """

    MOTHER = "32"
    """
    Mother
    """

    FATHER = "33"
    """
    Father
    """

    EMANCIPATED_MINOR = "36"
    """
    Emancipated Minor
    """

    ORGAN_DONOR = "39"
    """
    Organ Donor
    """

    CADAVER_DONOR = "40"
    """
    Cadaver Donor
    """

    INJURED_PLAINTIFF = "41"
    """
    Injured Plaintiff
    """

    CHILD_NO_FINANCIAL_RESPONSIBILITY = "43"
    """
    Child (No Financial Responsibility)
    """

    LIFE_PARTNER = "53"
    """
    Life Partner
    """

    OTHER_RELATIONSHIP = "G8"
    """
    Other Relationship
    """

    _UNKNOWN = "__PATIENTRELATIONSHIPTOINSUREDCODEALL_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "PatientRelationshipToInsuredCodeAll":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        spouse: typing.Callable[[], T_Result],
        grandparent: typing.Callable[[], T_Result],
        grandchild: typing.Callable[[], T_Result],
        niece_nephew: typing.Callable[[], T_Result],
        foster_child: typing.Callable[[], T_Result],
        ward_of_the_court: typing.Callable[[], T_Result],
        stepchild: typing.Callable[[], T_Result],
        self_: typing.Callable[[], T_Result],
        child: typing.Callable[[], T_Result],
        employee: typing.Callable[[], T_Result],
        unknown: typing.Callable[[], T_Result],
        handicapped_dependent: typing.Callable[[], T_Result],
        sponsored_dependent: typing.Callable[[], T_Result],
        dependent_of_minor_dependent: typing.Callable[[], T_Result],
        significant_other: typing.Callable[[], T_Result],
        mother: typing.Callable[[], T_Result],
        father: typing.Callable[[], T_Result],
        emancipated_minor: typing.Callable[[], T_Result],
        organ_donor: typing.Callable[[], T_Result],
        cadaver_donor: typing.Callable[[], T_Result],
        injured_plaintiff: typing.Callable[[], T_Result],
        child_no_financial_responsibility: typing.Callable[[], T_Result],
        life_partner: typing.Callable[[], T_Result],
        other_relationship: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is PatientRelationshipToInsuredCodeAll.SPOUSE:
            return spouse()
        if self is PatientRelationshipToInsuredCodeAll.GRANDPARENT:
            return grandparent()
        if self is PatientRelationshipToInsuredCodeAll.GRANDCHILD:
            return grandchild()
        if self is PatientRelationshipToInsuredCodeAll.NIECE_NEPHEW:
            return niece_nephew()
        if self is PatientRelationshipToInsuredCodeAll.FOSTER_CHILD:
            return foster_child()
        if self is PatientRelationshipToInsuredCodeAll.WARD_OF_THE_COURT:
            return ward_of_the_court()
        if self is PatientRelationshipToInsuredCodeAll.STEPCHILD:
            return stepchild()
        if self is PatientRelationshipToInsuredCodeAll.SELF:
            return self_()
        if self is PatientRelationshipToInsuredCodeAll.CHILD:
            return child()
        if self is PatientRelationshipToInsuredCodeAll.EMPLOYEE:
            return employee()
        if self is PatientRelationshipToInsuredCodeAll.UNKNOWN:
            return unknown()
        if self is PatientRelationshipToInsuredCodeAll.HANDICAPPED_DEPENDENT:
            return handicapped_dependent()
        if self is PatientRelationshipToInsuredCodeAll.SPONSORED_DEPENDENT:
            return sponsored_dependent()
        if self is PatientRelationshipToInsuredCodeAll.DEPENDENT_OF_MINOR_DEPENDENT:
            return dependent_of_minor_dependent()
        if self is PatientRelationshipToInsuredCodeAll.SIGNIFICANT_OTHER:
            return significant_other()
        if self is PatientRelationshipToInsuredCodeAll.MOTHER:
            return mother()
        if self is PatientRelationshipToInsuredCodeAll.FATHER:
            return father()
        if self is PatientRelationshipToInsuredCodeAll.EMANCIPATED_MINOR:
            return emancipated_minor()
        if self is PatientRelationshipToInsuredCodeAll.ORGAN_DONOR:
            return organ_donor()
        if self is PatientRelationshipToInsuredCodeAll.CADAVER_DONOR:
            return cadaver_donor()
        if self is PatientRelationshipToInsuredCodeAll.INJURED_PLAINTIFF:
            return injured_plaintiff()
        if self is PatientRelationshipToInsuredCodeAll.CHILD_NO_FINANCIAL_RESPONSIBILITY:
            return child_no_financial_responsibility()
        if self is PatientRelationshipToInsuredCodeAll.LIFE_PARTNER:
            return life_partner()
        if self is PatientRelationshipToInsuredCodeAll.OTHER_RELATIONSHIP:
            return other_relationship()
        return _unknown_member(self._value_)
